@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -32,14 +10,18 @@
 
 #include <cplusplus/CppDocument.h>
 #include <utils/environment.h>
+#include <utils/filepath.h>
 #include <utils/futuresynchronizer.h>
 #include <utils/qrcparser.h>
+#include <utils/synchronizedvalue.h>
 
 #include <QFuture>
 #include <QHash>
 #include <QObject>
 #include <QPointer>
 #include <QStringList>
+#include <QThreadPool>
+#include <QVersionNumber>
 
 QT_FORWARD_DECLARE_CLASS(QTimer)
 
@@ -67,12 +49,13 @@ public:
     struct ProjectInfo
     {
         QPointer<ProjectExplorer::Project> project;
-        QStringList sourceFiles;
+        QList<Utils::FilePath> sourceFiles;
         PathsAndLanguages importPaths;
-        QStringList activeResourceFiles;
-        QStringList allResourceFiles;
-        QHash<QString, QString> resourceFileContents;
-        QStringList applicationDirectories;
+        QList<Utils::FilePath> activeResourceFiles;
+        QList<Utils::FilePath> allResourceFiles;
+        QList<Utils::FilePath> generatedQrcFiles;
+        QHash<Utils::FilePath, QString> resourceFileContents;
+        QList<Utils::FilePath> applicationDirectories;
         QHash<QString, QString> moduleMappings; // E.g.: QtQuick.Controls -> MyProject.MyControls
 
         // whether trying to run qmldump makes sense
@@ -82,6 +65,7 @@ public:
         Utils::Environment qmlDumpEnvironment;
 
         Utils::FilePath qtQmlPath;
+        Utils::FilePath qmllsPath;
         QString qtVersionString;
         QmlJS::QmlLanguageBundles activeBundle;
         QmlJS::QmlLanguageBundles extendedBundle;
@@ -90,18 +74,18 @@ public:
     class WorkingCopy
     {
     public:
-        using Table = QHash<QString, QPair<QString, int>>;
+        using Table = QHash<Utils::FilePath, QPair<QString, int>>;
 
-        void insert(const QString &fileName, const QString &source, int revision = 0)
+        void insert(const Utils::FilePath &fileName, const QString &source, int revision = 0)
         { m_elements.insert(fileName, {source, revision}); }
 
-        bool contains(const QString &fileName) const
+        bool contains(const Utils::FilePath &fileName) const
         { return m_elements.contains(fileName); }
 
-        QString source(const QString &fileName) const
+        QString source(const Utils::FilePath &fileName) const
         { return m_elements.value(fileName).first; }
 
-        QPair<QString, int> get(const QString &fileName) const
+        QPair<QString, int> get(const Utils::FilePath &fileName) const
         { return m_elements.value(fileName); }
 
         Table all() const
@@ -123,22 +107,27 @@ public:
     ModelManagerInterface(QObject *parent = nullptr);
     ~ModelManagerInterface() override;
 
-    static Dialect guessLanguageOfFile(const QString &fileName);
+    static Dialect guessLanguageOfFile(const Utils::FilePath &fileName);
     static QStringList globPatternsForLanguages(const QList<Dialect> &languages);
     static ModelManagerInterface *instance();
     static ModelManagerInterface *instanceForFuture(const QFuture<void> &future);
     static void writeWarning(const QString &msg);
     static WorkingCopy workingCopy();
+    static Utils::FilePath qmllsForBinPath(const Utils::FilePath &binPath, const QVersionNumber &v);
 
     QmlJS::Snapshot snapshot() const;
     QmlJS::Snapshot newestSnapshot() const;
+    QThreadPool *threadPool();
+
+    QSet<Utils::FilePath> scannedPaths() const;
+    void removeFromScannedPaths(const PathsAndLanguages &pathsAndLanguages);
 
     void activateScan();
-    void updateSourceFiles(const QStringList &files,
-                           bool emitDocumentOnDiskChanged);
-    void fileChangedOnDisk(const QString &path);
-    void removeFiles(const QStringList &files);
-    QStringList qrcPathsForFile(const QString &file, const QLocale *locale = nullptr,
+    void updateSourceFiles(const QList<Utils::FilePath> &files, bool emitDocumentOnDiskChanged);
+    void fileChangedOnDisk(const Utils::FilePath &path);
+    void removeFiles(const QList<Utils::FilePath> &files);
+    QStringList qrcPathsForFile(const Utils::FilePath &file,
+                                const QLocale *locale = nullptr,
                                 ProjectExplorer::Project *project = nullptr,
                                 QrcResourceSelector resources = AllQrcResources);
     QStringList filesAtQrcPath(const QString &path, const QLocale *locale = nullptr,
@@ -149,6 +138,7 @@ public:
                                               ProjectExplorer::Project *project = nullptr,
                                               bool addDirs = false,
                                               QrcResourceSelector resources = AllQrcResources);
+    Utils::FilePath fileToSource(const Utils::FilePath &file);
 
     QList<ProjectInfo> projectInfos() const;
     bool containsProject(ProjectExplorer::Project *project) const;
@@ -158,16 +148,18 @@ public:
     void updateDocument(const QmlJS::Document::Ptr& doc);
     void updateLibraryInfo(const Utils::FilePath &path, const QmlJS::LibraryInfo &info);
     void emitDocumentChangedOnDisk(QmlJS::Document::Ptr doc);
-    void updateQrcFile(const QString &path);
-    ProjectInfo projectInfoForPath(const QString &path) const;
-    QList<ProjectInfo> allProjectInfosForPath(const QString &path) const;
+    void updateQrcFile(const Utils::FilePath &path);
+    ProjectInfo projectInfoForPath(const Utils::FilePath &path) const;
+    QList<ProjectInfo> allProjectInfosForPath(const Utils::FilePath &path) const;
 
-    QStringList importPathsNames() const;
+    QList<Utils::FilePath> importPathsNames() const;
     QmlJS::QmlLanguageBundles activeBundles() const;
     QmlJS::QmlLanguageBundles extendedBundles() const;
 
-    void loadPluginTypes(const QString &libraryPath, const QString &importPath,
-                         const QString &importUri, const QString &importVersion);
+    void loadPluginTypes(const Utils::FilePath &libraryPath,
+                         const Utils::FilePath &importPath,
+                         const QString &importUri,
+                         const QString &importVersion);
 
     CppDataHash cppData() const;
     LibraryInfo builtins(const Document::Ptr &doc) const;
@@ -180,8 +172,8 @@ public:
 
     void setDefaultVContext(const ViewerContext &vContext);
     virtual ProjectInfo defaultProjectInfo() const;
-    virtual ProjectInfo defaultProjectInfoForProject(ProjectExplorer::Project *project) const;
-
+    virtual ProjectInfo defaultProjectInfoForProject(ProjectExplorer::Project *project,
+                                                     const Utils::FilePaths &hiddenRccFolders) const;
 
     // Blocks until all parsing threads are done. Use for testing only!
     void test_joinAllThreads();
@@ -190,23 +182,28 @@ public:
     void addFuture(const QFuture<T> &future) { addFuture(QFuture<void>(future)); }
     void addFuture(const QFuture<void> &future);
 
-    QmlJS::Document::Ptr ensuredGetDocumentForPath(const QString &filePath);
-    static void importScan(QFutureInterface<void> &future, const WorkingCopy& workingCopyInternal,
-                           const PathsAndLanguages& paths, ModelManagerInterface *modelManager,
-                           bool emitDocChangedOnDisk, bool libOnly = true,
-                           bool forceRescan = false);
+    QmlJS::Document::Ptr ensuredGetDocumentForPath(const Utils::FilePath &filePath);
+    static void importScan(const WorkingCopy &workingCopy, const PathsAndLanguages &paths,
+                           ModelManagerInterface *modelManager, bool emitDocChanged,
+                           bool libOnly = true, bool forceRescan = false);
+    static void importScanAsync(QPromise<void> &promise, const WorkingCopy& workingCopyInternal,
+                                const PathsAndLanguages& paths, ModelManagerInterface *modelManager,
+                                bool emitDocChanged, bool libOnly = true, bool forceRescan = false);
 
     virtual void resetCodeModel();
     void removeProjectInfo(ProjectExplorer::Project *project);
     void maybeQueueCppQmlTypeUpdate(const CPlusPlus::Document::Ptr &doc);
 
+    QFuture<void> refreshSourceFiles(const QList<Utils::FilePath> &sourceFiles,
+                                     bool emitDocumentOnDiskChanged);
+
 signals:
     void documentUpdated(QmlJS::Document::Ptr doc);
     void documentChangedOnDisk(QmlJS::Document::Ptr doc);
-    void aboutToRemoveFiles(const QStringList &files);
-    void libraryInfoUpdated(const QString &path, const QmlJS::LibraryInfo &info);
+    void aboutToRemoveFiles(const QList<Utils::FilePath> &files);
+    void libraryInfoUpdated(const Utils::FilePath &path, const QmlJS::LibraryInfo &info);
     void projectInfoUpdated(const ProjectInfo &pinfo);
-    void projectPathChanged(const QString &projectPath);
+    void projectPathChanged(const Utils::FilePath &projectPath);
 
 protected:
     Q_INVOKABLE void queueCppQmlTypeUpdate(const CPlusPlus::Document::Ptr &doc, bool scan);
@@ -219,30 +216,29 @@ protected:
     virtual void addTaskInternal(const QFuture<void> &result, const QString &msg,
                                  const char *taskId) const;
 
-    QFuture<void> refreshSourceFiles(const QStringList &sourceFiles,
-                                     bool emitDocumentOnDiskChanged);
-
-    static void parseLoop(QSet<QString> &scannedPaths, QSet<QString> &newLibraries,
-                          const WorkingCopy &workingCopyInternal, QStringList files,
+    static void parseLoop(QSet<Utils::FilePath> &scannedPaths,
+                          QSet<Utils::FilePath> &newLibraries,
+                          const WorkingCopy &workingCopyInternal,
+                          QList<Utils::FilePath> files,
                           ModelManagerInterface *modelManager,
-                          QmlJS::Dialect mainLanguage, bool emitDocChangedOnDisk,
+                          QmlJS::Dialect mainLanguage,
+                          bool emitDocChangedOnDisk,
                           const std::function<bool(qreal)> &reportProgress);
-    static void parse(QFutureInterface<void> &future,
+    static void parse(QPromise<void> &promise,
                       const WorkingCopy &workingCopyInternal,
-                      QStringList files,
+                      QList<Utils::FilePath> files,
                       ModelManagerInterface *modelManager,
                       QmlJS::Dialect mainLanguage,
                       bool emitDocChangedOnDisk);
-    static void updateCppQmlTypes(
-            QFutureInterface<void> &futureInterface, ModelManagerInterface *qmlModelManager,
-            const CPlusPlus::Snapshot &snapshot,
-            const QHash<QString, QPair<CPlusPlus::Document::Ptr, bool>> &documents);
+    static void updateCppQmlTypes(QPromise<void> &promise, ModelManagerInterface *qmlModelManager,
+                const CPlusPlus::Snapshot &snapshot,
+                const QHash<QString, QPair<CPlusPlus::Document::Ptr, bool>> &documents);
 
     void maybeScan(const PathsAndLanguages &importPaths);
     void updateImportPaths();
     void loadQmlTypeDescriptionsInternal(const QString &path);
     void setDefaultProject(const ProjectInfo &pInfo, ProjectExplorer::Project *p);
-
+    void cancelAllThreads();
 private:
     void joinAllThreads(bool cancelOnWait = false);
     void iterateQrcFiles(ProjectExplorer::Project *project,
@@ -250,33 +246,39 @@ private:
                          const std::function<void(Utils::QrcParser::ConstPtr)> &callback);
     ViewerContext getVContext(const ViewerContext &vCtx, const Document::Ptr &doc, bool limitToProject) const;
 
-    mutable QMutex m_mutex;
-    QmlJS::Snapshot m_validSnapshot;
-    QmlJS::Snapshot m_newestSnapshot;
-    PathsAndLanguages m_allImportPaths;
-    QStringList m_defaultImportPaths;
-    QmlJS::QmlLanguageBundles m_activeBundles;
-    QmlJS::QmlLanguageBundles m_extendedBundles;
-    QHash<Dialect, QmlJS::ViewerContext> m_defaultVContexts;
-    bool m_shouldScanImports = false;
-    QSet<QString> m_scannedPaths;
+    struct SyncedData
+    {
+        SyncedData(const QList<Utils::FilePath> &defaultImportPaths);
+        QmlJS::Snapshot m_validSnapshot;
+        QmlJS::Snapshot m_newestSnapshot;
+        PathsAndLanguages m_allImportPaths;
+        QList<Utils::FilePath> m_applicationPaths;
+        QList<Utils::FilePath> m_defaultImportPaths;
+        QmlJS::QmlLanguageBundles m_activeBundles;
+        QmlJS::QmlLanguageBundles m_extendedBundles;
+        QHash<Dialect, QmlJS::ViewerContext> m_defaultVContexts;
+        bool m_shouldScanImports = false;
+        QSet<Utils::FilePath> m_scannedPaths;
+        ProjectExplorer::Project *m_defaultProject = nullptr;
+        ProjectInfo m_defaultProjectInfo;
+        QHash<ProjectExplorer::Project *, ProjectInfo> m_projects;
+        QMultiHash<Utils::FilePath, ProjectExplorer::Project *> m_fileToProject;
+    };
+    Utils::SynchronizedValue<SyncedData> m_syncedData;
 
     QTimer *m_updateCppQmlTypesTimer = nullptr;
     QTimer *m_asyncResetTimer = nullptr;
     QHash<QString, QPair<CPlusPlus::Document::Ptr, bool>> m_queuedCppDocuments;
     QFuture<void> m_cppQmlTypesUpdater;
     Utils::QrcCache m_qrcCache;
-    QHash<QString, QString> m_qrcContents;
+    QHash<Utils::FilePath, QString> m_qrcContents;
 
-    CppDataHash m_cppDataHash;
-    QHash<QString, QList<CPlusPlus::Document::Ptr>> m_cppDeclarationFiles;
-    mutable QMutex m_cppDataMutex;
-
-    // project integration
-    QMap<ProjectExplorer::Project *, ProjectInfo> m_projects;
-    ProjectInfo m_defaultProjectInfo;
-    ProjectExplorer::Project *m_defaultProject = nullptr;
-    QMultiHash<QString, ProjectExplorer::Project *> m_fileToProject;
+    struct SyncedCppData
+    {
+        CppDataHash m_cppDataHash;
+        QHash<QString, QList<CPlusPlus::Document::Ptr>> m_cppDeclarationFiles;
+    };
+    Utils::SynchronizedValue<SyncedCppData> m_syncedCppData;
 
     PluginDumper *m_pluginDumper = nullptr;
 
@@ -284,6 +286,38 @@ private:
     Utils::FutureSynchronizer m_futureSynchronizer;
 
     bool m_indexerDisabled = false;
+    QThreadPool m_threadPool;
+
+private:
+    QList<Utils::FilePath> importPathsNames(const SyncedData &lockedData) const;
+
+    static bool findNewQmlApplicationInPath(
+        const Utils::FilePath &path,
+        const Snapshot &snapshot,
+        ModelManagerInterface *modelManager,
+        QSet<Utils::FilePath> *newLibraries,
+        Utils::SynchronizedValue<ModelManagerInterface::SyncedData>::unique_lock &lock);
+
+    static void findNewLibraryImports(const Document::Ptr &doc,
+                                      const Snapshot &snapshot,
+                                      ModelManagerInterface *modelManager,
+                                      Utils::FilePaths *importedFiles,
+                                      QSet<Utils::FilePath> *scannedPaths,
+                                      QSet<Utils::FilePath> *newLibraries,
+                                      Utils::SynchronizedValue<SyncedData>::unique_lock *lock);
+
+    static bool findNewQmlLibraryInPath(const Utils::FilePath &path,
+                                        const Snapshot &snapshot,
+                                        ModelManagerInterface *modelManager,
+                                        QList<Utils::FilePath> *importedFiles,
+                                        QSet<Utils::FilePath> *scannedPaths,
+                                        QSet<Utils::FilePath> *newLibraries,
+                                        bool ignoreMissing,
+                                        Utils::SynchronizedValue<SyncedData>::unique_lock *lock);
+
+    void updateLibraryInfo(const Utils::FilePath &path,
+                           const QmlJS::LibraryInfo &info,
+                           Utils::SynchronizedValue<SyncedData>::unique_lock &lock);
 };
 
 } // namespace QmlJS
