@@ -1,45 +1,22 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "testconfiguration.h"
-#include "testoutputreader.h"
-#include "testrunconfiguration.h"
 
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/projectinfo.h>
+#include "itestframework.h"
+#include "testrunconfiguration.h"
 
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/buildsystem.h>
 #include <projectexplorer/buildtargetinfo.h>
 #include <projectexplorer/deploymentdata.h>
-#include <projectexplorer/environmentaspect.h>
-#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/kitaspects.h>
 #include <projectexplorer/runconfiguration.h>
-#include <projectexplorer/session.h>
+#include <projectexplorer/projectmanager.h>
 #include <projectexplorer/target.h>
 
+#include <utils/algorithm.h>
+
 #include <QLoggingCategory>
 
 static Q_LOGGING_CATEGORY(LOG, "qtc.autotest.testconfiguration", QtWarningMsg)
@@ -49,8 +26,50 @@ using namespace Utils;
 
 namespace Autotest {
 
+ITestConfiguration::ITestConfiguration(ITestBase *testBase)
+    : m_testBase(testBase)
+{
+}
+
+void ITestConfiguration::setWorkingDirectory(const FilePath &workingDirectory)
+{
+    m_runnable.workingDirectory = workingDirectory;
+}
+
+FilePath ITestConfiguration::workingDirectory() const
+{
+    if (!m_runnable.workingDirectory.isEmpty()) {
+        if (m_runnable.workingDirectory.isDir()) // ensure wanted working dir does exist
+            return m_runnable.workingDirectory.absoluteFilePath();
+    }
+
+    const FilePath executable = executableFilePath();
+    return executable.isEmpty() ? executable : executable.absolutePath();
+}
+
+bool ITestConfiguration::hasExecutable() const
+{
+    return !m_runnable.command.isEmpty();
+}
+
+FilePath ITestConfiguration::executableFilePath() const
+{
+    if (!hasExecutable())
+        return {};
+
+    const Environment env = m_runnable.environment.appliedToEnvironment(
+        m_runnable.command.executable().deviceEnvironment());
+
+    return m_runnable.command.executable().searchInDirectories(env.path());
+}
+
+Environment ITestConfiguration::filteredEnvironment(const Environment &original) const
+{
+    return original;
+}
+
 TestConfiguration::TestConfiguration(ITestFramework *framework)
-    : m_framework(framework)
+    : ITestConfiguration(framework)
 {
 }
 
@@ -69,24 +88,24 @@ static FilePath ensureExeEnding(const FilePath &file)
 {
     if (!HostOsInfo::isWindowsHost() || file.isEmpty() || file.toString().toLower().endsWith(".exe"))
         return file;
-    return FilePath::fromString(HostOsInfo::withExecutableSuffix(file.toString()));
+    return file.withExecutableSuffix();
 }
 
-void TestConfiguration::completeTestInformation(ProjectExplorer::RunConfiguration *rc,
+void TestConfiguration::completeTestInformation(RunConfiguration *rc,
                                                 TestRunMode runMode)
 {
     QTC_ASSERT(rc, return);
-    QTC_ASSERT(m_project, return);
+    QTC_ASSERT(project(), return);
 
     if (hasExecutable()) {
         qCDebug(LOG) << "Executable has been set already - not completing configuration again.";
         return;
     }
-    Project *project = SessionManager::startupProject();
-    if (!project || project != m_project)
+    Project *startupProject = ProjectManager::startupProject();
+    if (!startupProject || startupProject != project())
         return;
 
-    Target *target = project->activeTarget();
+    Target *target = startupProject->activeTarget();
     if (!target)
         return;
 
@@ -94,18 +113,18 @@ void TestConfiguration::completeTestInformation(ProjectExplorer::RunConfiguratio
         return;
 
     m_runnable = rc->runnable();
-    m_displayName = rc->displayName();
+    setDisplayName(rc->displayName());
 
     BuildTargetInfo targetInfo = rc->buildTargetInfo();
     if (!targetInfo.targetFilePath.isEmpty())
-        m_runnable.executable = ensureExeEnding(targetInfo.targetFilePath);
+        m_runnable.command.setExecutable(ensureExeEnding(targetInfo.targetFilePath));
 
-    QString buildBase;
+    FilePath buildBase;
     if (auto buildConfig = target->activeBuildConfiguration()) {
-        buildBase = buildConfig->buildDirectory().toString();
-        const QString projBase = m_project->projectDirectory().toString();
+        buildBase = buildConfig->buildDirectory();
+        const QString projBase = startupProject->projectDirectory().toString();
         if (m_projectFile.startsWith(projBase))
-            m_buildDir = QFileInfo(buildBase + m_projectFile.mid(projBase.length())).absolutePath();
+            m_buildDir = (buildBase / m_projectFile.toString().mid(projBase.length())).absolutePath();
     }
     if (runMode == TestRunMode::Debug || runMode == TestRunMode::DebugWithoutDeploy)
         m_runConfig = new Internal::TestRunConfiguration(rc->target(), this);
@@ -115,26 +134,25 @@ void TestConfiguration::completeTestInformation(TestRunMode runMode)
 {
     QTC_ASSERT(!m_projectFile.isEmpty(), return);
     QTC_ASSERT(!m_buildTargets.isEmpty(), return);
-    QTC_ASSERT(m_project, return);
+    QTC_ASSERT(project(), return);
 
     if (m_origRunConfig) {
         qCDebug(LOG) << "Using run configuration specified by user or found by first call";
         completeTestInformation(m_origRunConfig, runMode);
         if (hasExecutable()) {
-            qCDebug(LOG) << "Completed.\nRunnable:" << m_runnable.executable
-                         << "\nArgs:" << m_runnable.commandLineArguments
+            qCDebug(LOG) << "Completed.\nCommand:" << m_runnable.command.toUserOutput()
                          << "\nWorking directory:" << m_runnable.workingDirectory;
             return;
         }
         qCDebug(LOG) << "Failed to complete - using 'normal' way.";
     }
-    Project *project = SessionManager::startupProject();
-    if (!project || project != m_project) {
-        m_project = nullptr;
+    Project *startupProject = ProjectManager::startupProject();
+    if (!startupProject || startupProject != project()) {
+        setProject(nullptr);
         return;
     }
 
-    Target *target = project->activeTarget();
+    Target *target = startupProject->activeTarget();
     if (!target)
         return;
     qCDebug(LOG) << "ActiveTargetName\n    " << target->displayName();
@@ -167,12 +185,12 @@ void TestConfiguration::completeTestInformation(TestRunMode runMode)
     if (localExecutable.isEmpty())
         return;
 
-    QString buildBase;
+    FilePath buildBase;
     if (auto buildConfig = target->activeBuildConfiguration()) {
-        buildBase = buildConfig->buildDirectory().toString();
-        const QString projBase = project->projectDirectory().toString();
+        buildBase = buildConfig->buildDirectory();
+        const QString projBase = startupProject->projectDirectory().toString();
         if (m_projectFile.startsWith(projBase))
-            m_buildDir = QFileInfo(buildBase + m_projectFile.mid(projBase.length())).absolutePath();
+            m_buildDir = (buildBase / m_projectFile.toString().mid(projBase.length())).absolutePath();
     }
 
     // deployment information should get taken into account, but it pretty much seems as if
@@ -189,17 +207,17 @@ void TestConfiguration::completeTestInformation(TestRunMode runMode)
     QList<RunConfiguration *> runConfigurations = target->runConfigurations();
     runConfigurations.removeOne(target->activeRunConfiguration());
     runConfigurations.prepend(target->activeRunConfiguration());
-    for (RunConfiguration *runConfig : qAsConst(runConfigurations)) {
+    for (RunConfiguration *runConfig : std::as_const(runConfigurations)) {
         qCDebug(LOG) << "RunConfiguration" << runConfig->id();
         if (!isLocal(target)) { // TODO add device support
             qCDebug(LOG) << " Skipped as not being local";
             continue;
         }
 
-        const Runnable runnable = runConfig->runnable();
+        const ProcessRunData runnable = runConfig->runnable();
         // not the best approach - but depending on the build system and whether the executables
         // are going to get installed or not we have to soften the condition...
-        const FilePath currentExecutable = ensureExeEnding(runnable.executable);
+        const FilePath currentExecutable = ensureExeEnding(runnable.command.executable());
         const QString currentBST = runConfig->buildKey();
         qCDebug(LOG) << " CurrentExecutable" << currentExecutable;
         qCDebug(LOG) << " BST of RunConfig" << currentBST;
@@ -209,8 +227,8 @@ void TestConfiguration::completeTestInformation(TestRunMode runMode)
             qCDebug(LOG) << "  Using this RunConfig.";
             m_origRunConfig = runConfig;
             m_runnable = runnable;
-            m_runnable.executable = currentExecutable;
-            m_displayName = runConfig->displayName();
+            m_runnable.command.setExecutable(currentExecutable);
+            setDisplayName(runConfig->displayName());
             if (runMode == TestRunMode::Debug || runMode == TestRunMode::DebugWithoutDeploy)
                 m_runConfig = new Internal::TestRunConfiguration(target, this);
             break;
@@ -222,13 +240,13 @@ void TestConfiguration::completeTestInformation(TestRunMode runMode)
     // for this case try the original executable path of the BuildTargetInfo (the executable
     // before installation) to have at least something to execute
     if (!hasExecutable() && !localExecutable.isEmpty())
-        m_runnable.executable = localExecutable;
-    if (m_displayName.isEmpty() && hasExecutable()) {
+        m_runnable.command.setExecutable(localExecutable);
+    if (displayName().isEmpty() && hasExecutable()) {
         qCDebug(LOG) << "   Fallback";
         // we failed to find a valid runconfiguration - but we've got the executable already
         if (auto rc = target->activeRunConfiguration()) {
             if (isLocal(target)) { // FIXME for now only Desktop support
-                const Runnable runnable = rc->runnable();
+                const ProcessRunData runnable = rc->runnable();
                 m_runnable.environment = runnable.environment;
                 m_deducedConfiguration = true;
                 m_deducedFrom = rc->displayName();
@@ -241,8 +259,8 @@ void TestConfiguration::completeTestInformation(TestRunMode runMode)
         }
     }
 
-    if (m_displayName.isEmpty()) // happens e.g. when deducing the TestConfiguration or error
-        m_displayName = (*buildSystemTargets.begin());
+    if (displayName().isEmpty()) // happens e.g. when deducing the TestConfiguration or error
+        setDisplayName(*buildSystemTargets.begin());
 }
 
 /**
@@ -257,49 +275,14 @@ void TestConfiguration::setTestCases(const QStringList &testCases)
 {
     m_testCases.clear();
     m_testCases << testCases;
-    m_testCaseCount = m_testCases.size();
+    setTestCaseCount(m_testCases.size());
 }
 
-void TestConfiguration::setTestCaseCount(int count)
-{
-    m_testCaseCount = count;
-}
-
-void TestConfiguration::setExecutableFile(const QString &executableFile)
-{
-    m_runnable.executable = Utils::FilePath::fromString(executableFile);
-}
-
-void TestConfiguration::setProjectFile(const QString &projectFile)
+void TestConfiguration::setProjectFile(const FilePath &projectFile)
 {
     m_projectFile = projectFile;
 }
 
-void TestConfiguration::setWorkingDirectory(const QString &workingDirectory)
-{
-    m_runnable.workingDirectory = workingDirectory;
-}
-
-void TestConfiguration::setBuildDirectory(const QString &buildDirectory)
-{
-    m_buildDir = buildDirectory;
-}
-
-void TestConfiguration::setDisplayName(const QString &displayName)
-{
-    m_displayName = displayName;
-}
-
-void TestConfiguration::setEnvironment(const Utils::Environment &env)
-{
-    m_runnable.environment = env;
-}
-
-void TestConfiguration::setProject(Project *project)
-{
-    m_project = project;
-}
-
 void TestConfiguration::setInternalTarget(const QString &target)
 {
     m_buildTargets.clear();
@@ -316,54 +299,9 @@ void TestConfiguration::setOriginalRunConfiguration(RunConfiguration *runConfig)
     m_origRunConfig = runConfig;
 }
 
-QString TestConfiguration::executableFilePath() const
-{
-    if (!hasExecutable())
-        return QString();
-
-    QFileInfo commandFileInfo = m_runnable.executable.toFileInfo();
-    if (commandFileInfo.isExecutable() && commandFileInfo.path() != ".") {
-        return commandFileInfo.absoluteFilePath();
-    } else if (commandFileInfo.path() == "."){
-        QString fullCommandFileName = m_runnable.executable.toString();
-        // TODO: check if we can use searchInPath() from Utils::Environment
-        const QStringList &pathList = m_runnable.environment.toProcessEnvironment().value("PATH")
-                .split(Utils::HostOsInfo::pathListSeparator());
-
-        foreach (const QString &path, pathList) {
-            QString filePath(path + QDir::separator() + fullCommandFileName);
-            if (QFileInfo(filePath).isExecutable())
-                return commandFileInfo.absoluteFilePath();
-        }
-    }
-    return QString();
-}
-
-QString TestConfiguration::workingDirectory() const
-{
-    if (!m_runnable.workingDirectory.isEmpty()) {
-        const QFileInfo info(m_runnable.workingDirectory);
-        if (info.isDir()) // ensure wanted working dir does exist
-            return info.absoluteFilePath();
-    }
-
-    const QString executable = executableFilePath();
-    return executable.isEmpty() ? executable : QFileInfo(executable).absolutePath();
-}
-
 bool DebuggableTestConfiguration::isDebugRunMode() const
 {
     return m_runMode == TestRunMode::Debug || m_runMode == TestRunMode::DebugWithoutDeploy;
 }
 
-bool TestConfiguration::hasExecutable() const
-{
-    return !m_runnable.executable.isEmpty();
-}
-
-ITestFramework *TestConfiguration::framework() const
-{
-    return m_framework;
-}
-
 } // namespace Autotest
