@@ -1,12 +1,13 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "messageoutputwindow.h"
-#include "outputwindow.h"
-#include "icontext.h"
-#include "coreconstants.h"
 
-#include <coreplugin/icore.h>
+#include "coreconstants.h"
+#include "coreplugintr.h"
+#include "icontext.h"
+#include "outputwindow.h"
+
 #include <utils/utilsicons.h>
 
 #include <QFont>
@@ -19,6 +20,10 @@ const char zoomSettingsKey[] = "Core/MessageOutput/Zoom";
 
 MessageOutputWindow::MessageOutputWindow()
 {
+    setId("GeneralMessages");
+    setDisplayName(Tr::tr("General Messages"));
+    setPriorityInStatusBar(-100);
+
     m_widget = new OutputWindow(Context(Constants::C_GENERAL_OUTPUT_PANE), zoomSettingsKey);
     m_widget->setReadOnly(true);
 
@@ -64,21 +69,11 @@ QWidget *MessageOutputWindow::outputWidget(QWidget *parent)
     return m_widget;
 }
 
-QString MessageOutputWindow::displayName() const
-{
-    return tr("General Messages");
-}
-
 void MessageOutputWindow::append(const QString &text)
 {
     m_widget->appendMessage(text, Utils::GeneralMessageFormat);
 }
 
-int MessageOutputWindow::priorityInStatusBar() const
-{
-    return -1;
-}
-
 bool MessageOutputWindow::canNext() const
 {
     return false;
