@@ -1,25 +1,28 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "compileoutputwindow.h"
 
 #include "buildmanager.h"
-#include "ioutputparser.h"
-#include "projectexplorer.h"
+#include "projectexplorerconstants.h"
 #include "projectexplorericons.h"
 #include "projectexplorersettings.h"
+#include "projectexplorertr.h"
 #include "showoutputtaskhandler.h"
-#include "task.h"
-#include "taskhub.h"
 
 #include <coreplugin/outputwindow.h>
+#include <coreplugin/dialogs/ioptionspage.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/coreconstants.h>
+
 #include <extensionsystem/pluginmanager.h>
+
 #include <texteditor/texteditorsettings.h>
 #include <texteditor/fontsettings.h>
 #include <texteditor/behaviorsettings.h>
+
 #include <utils/algorithm.h>
+#include <utils/layoutbuilder.h>
 #include <utils/outputformatter.h>
 #include <utils/proxyaction.h>
 #include <utils/theme/theme.h>
@@ -37,20 +40,20 @@
 #include <QToolButton>
 #include <QVBoxLayout>
 
-namespace ProjectExplorer {
-namespace Internal {
+namespace ProjectExplorer::Internal {
 
 const char SETTINGS_KEY[] = "ProjectExplorer/CompileOutput/Zoom";
 const char C_COMPILE_OUTPUT[] = "ProjectExplorer.CompileOutput";
-const char POP_UP_KEY[] = "ProjectExplorer/Settings/ShowCompilerOutput";
-const char WRAP_OUTPUT_KEY[] = "ProjectExplorer/Settings/WrapBuildOutput";
-const char MAX_LINES_KEY[] = "ProjectExplorer/Settings/MaxBuildOutputLines";
 const char OPTIONS_PAGE_ID[] = "C.ProjectExplorer.CompileOutputOptions";
 
 CompileOutputWindow::CompileOutputWindow(QAction *cancelBuildAction) :
     m_cancelBuildButton(new QToolButton),
     m_settingsButton(new QToolButton)
 {
+    setId("CompileOutput");
+    setDisplayName(QCoreApplication::translate("QtC::ProjectExplorer", "Compile Output"));
+    setPriorityInStatusBar(40);
+
     Core::Context context(C_COMPILE_OUTPUT);
     m_outputWindow = new Core::OutputWindow(context, SETTINGS_KEY);
     m_outputWindow->setWindowTitle(displayName());
@@ -63,7 +66,7 @@ CompileOutputWindow::CompileOutputWindow(QAction *cancelBuildAction) :
             Utils::ProxyAction::proxyActionWithIcon(cancelBuildAction,
                                                     Utils::Icons::STOP_SMALL_TOOLBAR.icon());
     m_cancelBuildButton->setDefaultAction(cancelBuildProxyButton);
-    m_settingsButton->setToolTip(tr("Open Settings Page"));
+    m_settingsButton->setToolTip(Core::ICore::msgShowOptionsDialog());
     m_settingsButton->setIcon(Utils::Icons::SETTINGS_TOOLBAR.icon());
 
     auto updateFontSettings = [this] {
@@ -72,7 +75,7 @@ CompileOutputWindow::CompileOutputWindow(QAction *cancelBuildAction) :
 
     auto updateZoomEnabled = [this] {
         m_outputWindow->setWheelZoomEnabled(
-                    TextEditor::TextEditorSettings::behaviorSettings().m_scrollWheelZooming);
+                    TextEditor::globalBehaviorSettings().m_scrollWheelZooming);
     };
 
     updateFontSettings();
@@ -95,13 +98,23 @@ CompileOutputWindow::CompileOutputWindow(QAction *cancelBuildAction) :
     qRegisterMetaType<QTextCharFormat>("QTextCharFormat");
 
     m_handler = new ShowOutputTaskHandler(this,
-        tr("Show Compile &Output"),
-        tr("Show the output that generated this issue in Compile Output."),
-        tr("O"));
+        Tr::tr("Show Compile &Output"),
+        Tr::tr("Show the output that generated this issue in Compile Output."),
+        Tr::tr("O"));
     ExtensionSystem::PluginManager::addObject(m_handler);
     setupContext(C_COMPILE_OUTPUT, m_outputWindow);
-    loadSettings();
     updateFromSettings();
+
+    CompileOutputSettings &s = compileOutputSettings();
+    m_outputWindow->setWordWrapEnabled(s.wrapOutput());
+    m_outputWindow->setMaxCharCount(s.maxCharCount());
+
+    connect(&s.wrapOutput, &Utils::BaseAspect::changed, m_outputWindow, [this] {
+        m_outputWindow->setWordWrapEnabled(compileOutputSettings().wrapOutput());
+    });
+    connect(&s.maxCharCount, &Utils::BaseAspect::changed, m_outputWindow, [this] {
+        m_outputWindow->setMaxCharCount(compileOutputSettings().maxCharCount());
+    });
 }
 
 CompileOutputWindow::~CompileOutputWindow()
@@ -114,10 +127,7 @@ CompileOutputWindow::~CompileOutputWindow()
 
 void CompileOutputWindow::updateFromSettings()
 {
-    m_outputWindow->setWordWrapEnabled(m_settings.wrapOutput);
-    m_outputWindow->setMaxCharCount(m_settings.maxCharCount);
 }
-
 bool CompileOutputWindow::hasFocus() const
 {
     return m_outputWindow->window()->focusWidget() == m_outputWindow;
@@ -170,11 +180,6 @@ void CompileOutputWindow::clearContents()
     m_outputWindow->clear();
 }
 
-int CompileOutputWindow::priorityInStatusBar() const
-{
-    return 50;
-}
-
 bool CompileOutputWindow::canNext() const
 {
     return false;
@@ -212,13 +217,6 @@ void CompileOutputWindow::reset()
     m_outputWindow->reset();
 }
 
-void CompileOutputWindow::setSettings(const CompileOutputSettings &settings)
-{
-    m_settings = settings;
-    storeSettings();
-    updateFromSettings();
-}
-
 Utils::OutputFormatter *CompileOutputWindow::outputFormatter() const
 {
     return m_outputWindow->outputFormatter();
@@ -230,77 +228,60 @@ void CompileOutputWindow::updateFilter()
                                            filterUsesRegexp(), filterIsInverted());
 }
 
-const bool kPopUpDefault = false;
-const bool kWrapOutputDefault = true;
+// CompileOutputSettings
 
-void CompileOutputWindow::loadSettings()
+CompileOutputSettings &compileOutputSettings()
 {
-    QSettings * const s = Core::ICore::settings();
-    m_settings.popUp = s->value(POP_UP_KEY, kPopUpDefault).toBool();
-    m_settings.wrapOutput = s->value(WRAP_OUTPUT_KEY, kWrapOutputDefault).toBool();
-    m_settings.maxCharCount = s->value(MAX_LINES_KEY,
-                                       Core::Constants::DEFAULT_MAX_CHAR_COUNT).toInt() * 100;
+    static CompileOutputSettings theSettings;
+    return theSettings;
 }
 
-void CompileOutputWindow::storeSettings() const
+CompileOutputSettings::CompileOutputSettings()
 {
-    Utils::QtcSettings *const s = Core::ICore::settings();
-    s->setValueWithDefault(POP_UP_KEY, m_settings.popUp, kPopUpDefault);
-    s->setValueWithDefault(WRAP_OUTPUT_KEY, m_settings.wrapOutput, kWrapOutputDefault);
-    s->setValueWithDefault(MAX_LINES_KEY,
-                           m_settings.maxCharCount / 100,
-                           Core::Constants::DEFAULT_MAX_CHAR_COUNT);
-}
+    setAutoApply(false);
 
-class CompileOutputSettingsWidget : public Core::IOptionsPageWidget
-{
-    Q_DECLARE_TR_FUNCTIONS(ProjectExplorer::Internal::CompileOutputSettingsPage)
-public:
-    CompileOutputSettingsWidget()
-    {
-        const CompileOutputSettings &settings = BuildManager::compileOutputSettings();
-        m_wrapOutputCheckBox.setText(tr("Word-wrap output"));
-        m_wrapOutputCheckBox.setChecked(settings.wrapOutput);
-        m_popUpCheckBox.setText(tr("Open Compile Output when building"));
-        m_popUpCheckBox.setChecked(settings.popUp);
-        m_maxCharsBox.setMaximum(100000000);
-        m_maxCharsBox.setValue(settings.maxCharCount);
-        const auto layout = new QVBoxLayout(this);
-        layout->addWidget(&m_wrapOutputCheckBox);
-        layout->addWidget(&m_popUpCheckBox);
-        const auto maxCharsLayout = new QHBoxLayout;
-        const QString msg = tr("Limit output to %1 characters");
+    wrapOutput.setSettingsKey("ProjectExplorer/Settings/WrapBuildOutput");
+    wrapOutput.setDefaultValue(true);
+    wrapOutput.setLabelText(Tr::tr("Word-wrap output"));
+
+    popUp.setSettingsKey("ProjectExplorer/Settings/ShowCompilerOutput");
+    popUp.setLabelText(Tr::tr("Open Compile Output when building"));
+
+    maxCharCount.setSettingsKey("ProjectExplorer/Settings/MaxBuildOutputLines");
+    maxCharCount.setRange(1, Core::Constants::DEFAULT_MAX_CHAR_COUNT);
+    maxCharCount.setDefaultValue(Core::Constants::DEFAULT_MAX_CHAR_COUNT);
+    maxCharCount.setToSettingsTransformation([](const QVariant &v) { return v.toInt() / 100; });
+    maxCharCount.setFromSettingsTransformation([](const QVariant &v) { return v.toInt() * 100; });
+
+    setLayouter([this] {
+        using namespace Layouting;
+        const QString msg = Tr::tr("Limit output to %1 characters");
         const QStringList parts = msg.split("%1") << QString() << QString();
-        maxCharsLayout->addWidget(new QLabel(parts.at(0).trimmed()));
-        maxCharsLayout->addWidget(&m_maxCharsBox);
-        maxCharsLayout->addWidget(new QLabel(parts.at(1).trimmed()));
-        maxCharsLayout->addStretch(1);
-        layout->addLayout(maxCharsLayout);
-        layout->addStretch(1);
-    }
+        return Column {
+            wrapOutput,
+            popUp,
+            Row { parts.at(0), maxCharCount, parts.at(1), st },
+            st
+        };
+    });
 
-    void apply() final
+    readSettings();
+}
+
+// CompileOutputSettingsPage
+
+class CompileOutputSettingsPage final : public Core::IOptionsPage
+{
+public:
+    CompileOutputSettingsPage()
     {
-        CompileOutputSettings s;
-        s.wrapOutput = m_wrapOutputCheckBox.isChecked();
-        s.popUp = m_popUpCheckBox.isChecked();
-        s.maxCharCount = m_maxCharsBox.value();
-        BuildManager::setCompileOutputSettings(s);
+        setId(OPTIONS_PAGE_ID);
+        setDisplayName(Tr::tr("Compile Output"));
+        setCategory(Constants::BUILD_AND_RUN_SETTINGS_CATEGORY);
+        setSettingsProvider([] { return &compileOutputSettings(); });
     }
-
-private:
-    QCheckBox m_wrapOutputCheckBox;
-    QCheckBox m_popUpCheckBox;
-    QSpinBox m_maxCharsBox;
 };
 
-CompileOutputSettingsPage::CompileOutputSettingsPage()
-{
-    setId(OPTIONS_PAGE_ID);
-    setDisplayName(CompileOutputSettingsWidget::tr("Compile Output"));
-    setCategory(Constants::BUILD_AND_RUN_SETTINGS_CATEGORY);
-    setWidgetCreator([] { return new CompileOutputSettingsWidget; });
-}
+const CompileOutputSettingsPage settingsPage;
 
-} // Internal
-} // ProjectExplorer
+} // ProjectExplorer::Internal
