@@ -1,33 +1,27 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "devicemanager.h"
 
 #include "idevicefactory.h"
+#include "../projectexplorertr.h"
+#include "../projectexplorerconstants.h"
 
 #include <coreplugin/icore.h>
-#include <coreplugin/messagemanager.h>
-
-#include <projectexplorer/projectexplorerconstants.h>
 
 #include <utils/algorithm.h>
 #include <utils/devicefileaccess.h>
 #include <utils/environment.h>
-#include <utils/fileutils.h>
 #include <utils/fsengine/fsengine.h>
 #include <utils/persistentsettings.h>
-#include <utils/portlist.h>
+#include <utils/process.h>
 #include <utils/qtcassert.h>
-#include <utils/qtcprocess.h>
 #include <utils/stringutils.h>
+#include <utils/terminalhooks.h>
 
-#include <QDateTime>
-#include <QFileInfo>
 #include <QHash>
-#include <QList>
 #include <QMutex>
 #include <QMutexLocker>
-#include <QString>
 #include <QVariantList>
 
 #include <limits>
@@ -42,14 +36,12 @@ const char DeviceManagerKey[] = "DeviceManager";
 const char DeviceListKey[] = "DeviceList";
 const char DefaultDevicesKey[] = "DefaultDevices";
 
-template <class ...Args> using Continuation = std::function<void(Args...)>;
-
 class DeviceManagerPrivate
 {
 public:
     DeviceManagerPrivate() = default;
 
-    int indexForId(Utils::Id id) const
+    int indexForId(Id id) const
     {
         for (int i = 0; i < devices.count(); ++i) {
             if (devices.at(i)->id() == id)
@@ -68,8 +60,8 @@ public:
 
     mutable QMutex mutex;
     QList<IDevice::Ptr> devices;
-    QHash<Utils::Id, Utils::Id> defaultDevices;
-    Utils::PersistentSettingsWriter *writer = nullptr;
+    QHash<Id, Id> defaultDevices;
+    PersistentSettingsWriter *writer = nullptr;
 };
 DeviceManager *DeviceManagerPrivate::clonedInstance = nullptr;
 
@@ -123,6 +115,11 @@ DeviceManager *DeviceManager::cloneInstance()
     return DeviceManagerPrivate::clonedInstance;
 }
 
+DeviceManager *DeviceManager::clonedInstance()
+{
+    return DeviceManagerPrivate::clonedInstance;
+}
+
 void DeviceManager::copy(const DeviceManager *source, DeviceManager *target, bool deep)
 {
     if (deep) {
@@ -138,8 +135,8 @@ void DeviceManager::save()
 {
     if (d->clonedInstance == this || !d->writer)
         return;
-    QVariantMap data;
-    data.insert(QLatin1String(DeviceManagerKey), toMap());
+    Store data;
+    data.insert(DeviceManagerKey, variantFromStore(toMap()));
     d->writer->save(data, Core::ICore::dialogParent());
 }
 
@@ -160,16 +157,16 @@ void DeviceManager::load()
     // Only create writer now: We do not want to save before the settings were read!
     d->writer = new PersistentSettingsWriter(settingsFilePath("devices.xml"), "QtCreatorDevices");
 
-    Utils::PersistentSettingsReader reader;
+    PersistentSettingsReader reader;
     // read devices file from global settings path
-    QHash<Utils::Id, Utils::Id> defaultDevices;
+    QHash<Id, Id> defaultDevices;
     QList<IDevice::Ptr> sdkDevices;
     if (reader.load(systemSettingsFilePath("devices.xml")))
-        sdkDevices = fromMap(reader.restoreValues().value(DeviceManagerKey).toMap(), &defaultDevices);
+        sdkDevices = fromMap(storeFromVariant(reader.restoreValues().value(DeviceManagerKey)), &defaultDevices);
     // read devices file from user settings path
     QList<IDevice::Ptr> userDevices;
     if (reader.load(settingsFilePath("devices.xml")))
-        userDevices = fromMap(reader.restoreValues().value(DeviceManagerKey).toMap(), &defaultDevices);
+        userDevices = fromMap(storeFromVariant(reader.restoreValues().value(DeviceManagerKey)), &defaultDevices);
     // Insert devices into the model. Prefer the higher device version when there are multiple
     // devices with the same id.
     for (IDevice::ConstPtr device : std::as_const(userDevices)) {
@@ -197,9 +194,9 @@ void DeviceManager::load()
     emit devicesLoaded();
 }
 
-static const IDeviceFactory *restoreFactory(const QVariantMap &map)
+static const IDeviceFactory *restoreFactory(const Store &map)
 {
-    const Utils::Id deviceType = IDevice::typeFromMap(map);
+    const Id deviceType = IDevice::typeFromMap(map);
     IDeviceFactory *factory = Utils::findOrDefault(IDeviceFactory::allDeviceFactories(),
         [&map, deviceType](IDeviceFactory *factory) {
             return factory->canRestore(map) && factory->deviceType() == deviceType;
@@ -212,19 +209,18 @@ static const IDeviceFactory *restoreFactory(const QVariantMap &map)
     return factory;
 }
 
-QList<IDevice::Ptr> DeviceManager::fromMap(const QVariantMap &map,
-                                           QHash<Utils::Id, Utils::Id> *defaultDevices)
+QList<IDevice::Ptr> DeviceManager::fromMap(const Store &map, QHash<Id, Id> *defaultDevices)
 {
     QList<IDevice::Ptr> devices;
 
     if (defaultDevices) {
-        const QVariantMap defaultDevsMap = map.value(DefaultDevicesKey).toMap();
+        const Store defaultDevsMap = storeFromVariant(map.value(DefaultDevicesKey));
         for (auto it = defaultDevsMap.constBegin(); it != defaultDevsMap.constEnd(); ++it)
-            defaultDevices->insert(Utils::Id::fromString(it.key()), Utils::Id::fromSetting(it.value()));
+            defaultDevices->insert(Id::fromString(stringFromKey(it.key())), Id::fromSetting(it.value()));
     }
-    const QVariantList deviceList = map.value(QLatin1String(DeviceListKey)).toList();
+    const QVariantList deviceList = map.value(DeviceListKey).toList();
     for (const QVariant &v : deviceList) {
-        const QVariantMap map = v.toMap();
+        const Store map = storeFromVariant(v);
         const IDeviceFactory * const factory = restoreFactory(map);
         if (!factory)
             continue;
@@ -236,20 +232,18 @@ QList<IDevice::Ptr> DeviceManager::fromMap(const QVariantMap &map,
     return devices;
 }
 
-QVariantMap DeviceManager::toMap() const
+Store DeviceManager::toMap() const
 {
-    QVariantMap map;
-    QVariantMap defaultDeviceMap;
-    using TypeIdHash = QHash<Utils::Id, Utils::Id>;
-    for (TypeIdHash::ConstIterator it = d->defaultDevices.constBegin();
-             it != d->defaultDevices.constEnd(); ++it) {
-        defaultDeviceMap.insert(it.key().toString(), it.value().toSetting());
-    }
-    map.insert(QLatin1String(DefaultDevicesKey), defaultDeviceMap);
+    Store map;
+    Store defaultDeviceMap;
+    for (auto it = d->defaultDevices.constBegin(); it != d->defaultDevices.constEnd(); ++it)
+        defaultDeviceMap.insert(keyFromString(it.key().toString()), it.value().toSetting());
+
+    map.insert(DefaultDevicesKey, variantFromStore(defaultDeviceMap));
     QVariantList deviceList;
     for (const IDevice::Ptr &device : std::as_const(d->devices))
-        deviceList << device->toMap();
-    map.insert(QLatin1String(DeviceListKey), deviceList);
+        deviceList << variantFromStore(device->toMap());
+    map.insert(DeviceListKey, deviceList);
     return map;
 }
 
@@ -264,7 +258,8 @@ void DeviceManager::addDevice(const IDevice::ConstPtr &_device)
     }
 
     // TODO: make it thread safe?
-    device->setDisplayName(Utils::makeUniquelyNumbered(device->displayName(), names));
+    device->settings()->displayName.setValue(
+        Utils::makeUniquelyNumbered(device->displayName(), names));
 
     const int pos = d->indexForId(device->id());
 
@@ -286,31 +281,29 @@ void DeviceManager::addDevice(const IDevice::ConstPtr &_device)
         }
         emit deviceAdded(device->id());
 
-        if (FSEngine::isAvailable()) {
-            Utils::FSEngine::addDevice(device->rootPath());
-        }
+        if (FSEngine::isAvailable())
+            FSEngine::addDevice(device->rootPath());
     }
 
     emit updated();
 }
 
-void DeviceManager::removeDevice(Utils::Id id)
+void DeviceManager::removeDevice(Id id)
 {
     const IDevice::Ptr device = mutableDevice(id);
     QTC_ASSERT(device, return);
     QTC_ASSERT(this != instance() || device->isAutoDetected(), return);
 
     const bool wasDefault = d->defaultDevices.value(device->type()) == device->id();
-    const Utils::Id deviceType = device->type();
+    const Id deviceType = device->type();
     {
         QMutexLocker locker(&d->mutex);
         d->devices.removeAt(d->indexForId(id));
     }
     emit deviceRemoved(device->id());
 
-    if (FSEngine::isAvailable()) {
-        Utils::FSEngine::removeDevice(device->rootPath());
-    }
+    if (FSEngine::isAvailable())
+        FSEngine::removeDevice(device->rootPath());
 
     if (wasDefault) {
         for (int i = 0; i < d->devices.count(); ++i) {
@@ -327,7 +320,7 @@ void DeviceManager::removeDevice(Utils::Id id)
     emit updated();
 }
 
-void DeviceManager::setDeviceState(Utils::Id deviceId, IDevice::DeviceState deviceState)
+void DeviceManager::setDeviceState(Id deviceId, IDevice::DeviceState deviceState)
 {
     // To see the state change in the DeviceSettingsWidget. This has to happen before
     // the pos check below, in case the device is only present in the cloned instance.
@@ -378,7 +371,7 @@ IDevice::ConstPtr DeviceManager::defaultDesktopDevice()
     return m_instance->defaultDevice(Constants::DESKTOP_DEVICE_TYPE);
 }
 
-void DeviceManager::setDefaultDevice(Utils::Id id)
+void DeviceManager::setDefaultDevice(Id id)
 {
     QTC_ASSERT(this != instance(), return);
 
@@ -414,24 +407,43 @@ DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManager
         return leftDevice == rightDevice;
     };
 
-    deviceHooks.fileAccess = [](const FilePath &filePath) -> DeviceFileAccess * {
-        if (!filePath.needsDevice())
-            return DesktopDeviceFileAccess::instance();
-        auto device = DeviceManager::deviceForPath(filePath);
-        QTC_ASSERT(device, return nullptr);
-        return device->fileAccess();
+    deviceHooks.localSource = [](const FilePath &file) -> expected_str<FilePath> {
+        auto device = DeviceManager::deviceForPath(file);
+        if (!device)
+            return make_unexpected(Tr::tr("No device for path \"%1\"").arg(file.toUserOutput()));
+        return device->localSource(file);
     };
 
-    deviceHooks.environment = [](const FilePath &filePath) {
+    deviceHooks.fileAccess = [](const FilePath &filePath) -> expected_str<DeviceFileAccess *> {
+        if (!filePath.needsDevice())
+            return DesktopDeviceFileAccess::instance();
+        IDevice::ConstPtr device = DeviceManager::deviceForPath(filePath);
+        if (!device) {
+            return make_unexpected(
+                Tr::tr("No device found for path \"%1\"").arg(filePath.toUserOutput()));
+        }
+        DeviceFileAccess *fileAccess = device->fileAccess();
+        if (!fileAccess) {
+            return make_unexpected(
+                Tr::tr("No file access for device \"%1\"").arg(device->displayName()));
+        }
+        return fileAccess;
+    };
+
+    deviceHooks.environment = [](const FilePath &filePath) -> expected_str<Environment> {
         auto device = DeviceManager::deviceForPath(filePath);
-        QTC_ASSERT(device, return Environment{});
-        return device->systemEnvironment();
+        if (!device) {
+            return make_unexpected(
+                Tr::tr("No device found for path \"%1\"").arg(filePath.toUserOutput()));
+        }
+        return device->systemEnvironmentWithError();
     };
 
     deviceHooks.deviceDisplayName = [](const FilePath &filePath) {
         auto device = DeviceManager::deviceForPath(filePath);
-        QTC_ASSERT(device, return QString());
-        return device->displayName();
+        if (device)
+            return device->displayName();
+        return filePath.host().toString();
     };
 
     deviceHooks.ensureReachable = [](const FilePath &filePath, const FilePath &other) {
@@ -440,6 +452,19 @@ DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManager
         return device->ensureReachable(other);
     };
 
+    deviceHooks.openTerminal = [](const FilePath &filePath, const Environment &env) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return);
+        device->openTerminal(env, filePath);
+    };
+
+    deviceHooks.osType = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        if (!device)
+            return OsTypeLinux;
+        return device->osType();
+    };
+
     DeviceProcessHooks processHooks;
 
     processHooks.processImplHook = [](const FilePath &filePath) -> ProcessInterface * {
@@ -448,13 +473,7 @@ DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManager
         return device->createProcessInterface();
     };
 
-    processHooks.systemEnvironmentForBinary = [](const FilePath &filePath) {
-        auto device = DeviceManager::deviceForPath(filePath);
-        QTC_ASSERT(device, return Environment());
-        return device->systemEnvironment();
-    };
-
-    QtcProcess::setRemoteProcessHooks(processHooks);
+    Process::setRemoteProcessHooks(processHooks);
 }
 
 DeviceManager::~DeviceManager()
@@ -471,7 +490,15 @@ IDevice::ConstPtr DeviceManager::deviceAt(int idx) const
     return d->devices.at(idx);
 }
 
-IDevice::Ptr DeviceManager::mutableDevice(Utils::Id id) const
+void DeviceManager::forEachDevice(const std::function<void(const IDeviceConstPtr &)> &func) const
+{
+    const QList<IDevice::Ptr> devices = d->deviceList();
+
+    for (const IDevice::Ptr &device : devices)
+        func(device);
+}
+
+IDevice::Ptr DeviceManager::mutableDevice(Id id) const
 {
     const int index = d->indexForId(id);
     return index == -1 ? IDevice::Ptr() : d->devices.at(index);
@@ -484,15 +511,15 @@ bool DeviceManager::hasDevice(const QString &name) const
     });
 }
 
-IDevice::ConstPtr DeviceManager::find(Utils::Id id) const
+IDevice::ConstPtr DeviceManager::find(Id id) const
 {
     const int index = d->indexForId(id);
     return index == -1 ? IDevice::ConstPtr() : deviceAt(index);
 }
 
-IDevice::ConstPtr DeviceManager::defaultDevice(Utils::Id deviceType) const
+IDevice::ConstPtr DeviceManager::defaultDevice(Id deviceType) const
 {
-    const Utils::Id id = d->defaultDevices.value(deviceType);
+    const Id id = d->defaultDevices.value(deviceType);
     return id.isValid() ? find(id) : IDevice::ConstPtr();
 }
 
@@ -500,7 +527,7 @@ IDevice::ConstPtr DeviceManager::defaultDevice(Utils::Id deviceType) const
 
 
 #ifdef WITH_TESTS
-#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectexplorer_test.h>
 #include <QSignalSpy>
 #include <QTest>
 #include <QUuid>
@@ -512,20 +539,17 @@ class TestDevice : public IDevice
 public:
     TestDevice()
     {
-        setupId(AutoDetected, Utils::Id::fromString(QUuid::createUuid().toString()));
+        setupId(AutoDetected, Id::fromString(QUuid::createUuid().toString()));
         setType(testTypeId());
         setMachineType(Hardware);
         setOsType(HostOsInfo::hostOs());
         setDisplayType("blubb");
     }
 
-    static Utils::Id testTypeId() { return "TestType"; }
+    static Id testTypeId() { return "TestType"; }
+
 private:
     IDeviceWidget *createWidget() override { return nullptr; }
-    DeviceProcessSignalOperation::Ptr signalOperation() const override
-    {
-        return DeviceProcessSignalOperation::Ptr();
-    }
 };
 
 class TestDeviceFactory final : public IDeviceFactory
@@ -537,12 +561,12 @@ public:
     }
 };
 
-void ProjectExplorerPlugin::testDeviceManager()
+void ProjectExplorerTest::testDeviceManager()
 {
     TestDeviceFactory factory;
 
     TestDevice::Ptr dev = IDevice::Ptr(new TestDevice);
-    dev->setDisplayName(QLatin1String("blubbdiblubbfurz!"));
+    dev->settings()->displayName.setValue(QLatin1String("blubbdiblubbfurz!"));
     QVERIFY(dev->isAutoDetected());
     QCOMPARE(dev->deviceState(), IDevice::DeviceStateUnknown);
     QCOMPARE(dev->type(), TestDevice::testTypeId());
@@ -603,7 +627,7 @@ void ProjectExplorerPlugin::testDeviceManager()
     TestDevice::Ptr dev3 = IDevice::Ptr(new TestDevice);
     QVERIFY(dev->id() != dev3->id());
 
-    dev3->setDisplayName(dev->displayName());
+    dev3->settings()->displayName.setValue(dev->displayName());
     mgr->addDevice(dev3);
     QCOMPARE(mgr->deviceAt(mgr->deviceCount() - 1)->displayName(),
              QString(dev3->displayName() + QLatin1Char('2')));
