@@ -1,35 +1,13 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Alexander Drozdov.
-** Contact: Alexander Drozdov (adrozdoff@gmail.com)
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 Alexander Drozdov.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
 #include "projectexplorer_export.h"
 #include "projectnodes.h"
 
-#include <utils/mimetypes/mimedatabase.h>
-#include <utils/fileutils.h>
+#include <utils/filepath.h>
+#include <utils/mimeutils.h>
 
 #include <QObject>
 #include <QFuture>
@@ -53,7 +31,7 @@ public:
     };
     using Future = QFuture<Result>;
     using FutureWatcher = QFutureWatcher<Result>;
-    using FutureInterface = QFutureInterface<Result>;
+    using Promise = QPromise<Result>;
 
     using FileFilter = std::function<bool(const Utils::MimeType &, const Utils::FilePath &)>;
     using FileTypeFactory = std::function<ProjectExplorer::FileType(const Utils::MimeType &, const Utils::FilePath &)>;
@@ -91,7 +69,7 @@ signals:
     void finished();
 
 private:
-    static void scanForFiles(FutureInterface &fi, const Utils::FilePath &directory,
+    static void scanForFiles(Promise &fi, const Utils::FilePath &directory,
                              const FileFilter &filter, const FileTypeFactory &factory);
 
 private:
