@@ -1,17 +1,18 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
-
-#include "qtsupportplugin.h"
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "codegenerator.h"
-#include "codegensettingspage.h"
+#include "externaleditors.h"
 #include "gettingstartedwelcomepage.h"
 #include "profilereader.h"
 #include "qscxmlcgenerator.h"
-#include "qtkitinformation.h"
+#include "qtkitaspect.h"
 #include "qtoptionspage.h"
 #include "qtoutputformatter.h"
+#include "qtparser.h"
+#include "qtprojectimporter.h"
 #include "qtsupporttr.h"
+#include "qttestparser.h"
 #include "qtversionmanager.h"
 #include "qtversions.h"
 #include "translationwizardpage.h"
@@ -20,69 +21,130 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/jsexpander.h>
 
+#include <extensionsystem/iplugin.h>
+
 #include <projectexplorer/jsonwizard/jsonwizardfactory.h>
+#include <projectexplorer/buildpropertiessettings.h>
 #include <projectexplorer/project.h>
-#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projecttree.h>
-#include <projectexplorer/runcontrol.h>
-#include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
 
+#include <proparser/qmakeevaluator.h>
+
 #include <utils/filepath.h>
 #include <utils/infobar.h>
 #include <utils/macroexpander.h>
+#include <utils/process.h>
+
+#include <QInputDialog>
 
 using namespace Core;
 using namespace Utils;
 using namespace ProjectExplorer;
 
-namespace QtSupport {
-namespace Internal {
+namespace QtSupport::Internal {
 
-class QtSupportPluginPrivate
+static void processRunnerCallback(ProcessData *data)
 {
-public:
-    QtVersionManager qtVersionManager;
+    FilePath rootPath = FilePath::fromString(data->deviceRoot);
 
-    DesktopQtVersionFactory desktopQtVersionFactory;
-    EmbeddedLinuxQtVersionFactory embeddedLinuxQtVersionFactory;
+    Process proc;
+    proc.setProcessChannelMode(data->processChannelMode);
+    proc.setCommand({rootPath.withNewPath("/bin/sh"), {QString("-c"), data->command}});
+    proc.setWorkingDirectory(FilePath::fromString(data->workingDirectory));
+    proc.setEnvironment(Environment(data->environment.toStringList(), OsTypeLinux));
 
-    CodeGenSettingsPage codeGenSettingsPage;
-    QtOptionsPage qtOptionsPage;
+    proc.runBlocking();
 
-    ExamplesWelcomePage examplesPage{true};
-    ExamplesWelcomePage tutorialPage{false};
-
-    QtKitAspect qtKiAspect;
-
-    QtOutputFormatterFactory qtOutputFormatterFactory;
-
-    UicGeneratorFactory uicGeneratorFactory;
-    QScxmlcGeneratorFactory qscxmlcGeneratorFactory;
-};
-
-QtSupportPlugin::~QtSupportPlugin()
-{
-    delete d;
+    data->exitCode = proc.exitCode();
+    data->exitStatus = proc.exitStatus();
+    data->stdErr = proc.rawStdErr();
+    data->stdOut = proc.rawStdOut();
 }
 
-bool QtSupportPlugin::initialize(const QStringList &arguments, QString *errorMessage)
+class QtSupportPlugin final : public ExtensionSystem::IPlugin
 {
-    Q_UNUSED(arguments)
-    Q_UNUSED(errorMessage)
+    Q_OBJECT
+    Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QtCreatorPlugin" FILE "QtSupport.json")
+
+    void initialize() final;
+    void extensionsInitialized() final;
+    ShutdownFlag aboutToShutdown() final;
+};
+
+void QtSupportPlugin::initialize()
+{
+#ifdef WITH_TESTS
+    addTestCreator(createQtOutputFormatterTest);
+    addTestCreator(createQtBuildStringParserTest);
+    addTestCreator(createQtOutputParserTest);
+    addTestCreator(createQtTestParserTest);
+    addTestCreator(createQtProjectImporterTest);
+#endif
+
+    setupQtVersionManager(this);
+
+    setupDesktopQtVersion();
+    setupEmbeddedLinuxQtVersion();
+    setupGettingStartedWelcomePage();
+    setupQtSettingsPage();
+    setupQtOutputFormatter();
+    setupUicGenerator(this);
+    setupQScxmlcGenerator(this);
+
+    setupExternalDesigner(this);
+    setupExternalLinguist();
+
+    setupTranslationWizardPage();
+
+    theProcessRunner() = processRunnerCallback;
+
+    thePrompter() = [this](const QString &msg, const QStringList &context) -> std::optional<QString> {
+        std::optional<QString> res;
+        QEventLoop loop;
+
+        QMetaObject::invokeMethod(this, [msg, context, &res, &loop] {
+            QString text;
+            if (!context.isEmpty()) {
+                text = "Preceding lines:<i><br>&nbsp;&nbsp;&nbsp;..."
+                       + context.join("<br>&nbsp;&nbsp;&nbsp;")
+                       + "</i><p>";
+            }
+            text += msg;
+            bool ok = false;
+            const QString line = QInputDialog::getText(
+                ICore::dialogParent(),
+                /*title*/ "QMake Prompt",
+                /*label*/ text,
+                /*echo mode*/ QLineEdit::Normal,
+                /*text*/ QString(),
+                /*ok*/ &ok,
+                /*flags*/ Qt::WindowFlags(),
+                /*QInputMethodHints*/ Qt::ImhNone);
+            if (ok)
+                res = line;
+            loop.quit();
+        }, Qt::QueuedConnection);
+        loop.exec(QEventLoop::ExcludeUserInputEvents);
+        return res;
+    };
+
     QMakeParser::initialize();
     ProFileEvaluator::initialize();
     new ProFileCacheManager(this);
 
     JsExpander::registerGlobalObject<CodeGenerator>("QtSupport");
-    ProjectExplorer::JsonWizardFactory::registerPageFactory(new TranslationWizardPageFactory);
-    ProjectExplorerPlugin::showQtSettings();
 
-    d = new QtSupportPluginPrivate;
+    BuildPropertiesSettings::showQtSettings();
 
     QtVersionManager::initialized();
+}
 
-    return true;
+ExtensionSystem::IPlugin::ShutdownFlag QtSupportPlugin::aboutToShutdown()
+{
+    QtVersionManager::shutdown();
+    return SynchronousShutdown;
 }
 
 const char kLinkWithQtInstallationSetting[] = "LinkWithQtInstallation";
@@ -91,7 +153,7 @@ static void askAboutQtInstallation()
 {
     // if the install settings exist, the Qt Creator installation is (probably) already linked to
     // a Qt installation, so don't ask
-    if (!QtOptionsPage::canLinkWithQt() || QtOptionsPage::isLinkedWithQt()
+    if (!LinkWithQtSupport::canLinkWithQt() || LinkWithQtSupport::isLinkedWithQt()
         || !ICore::infoBar()->canInfoBeAdded(kLinkWithQtInstallationSetting))
         return;
 
@@ -103,7 +165,7 @@ static void askAboutQtInstallation()
         Utils::InfoBarEntry::GlobalSuppression::Enabled);
     info.addCustomButton(Tr::tr("Link with Qt"), [] {
         ICore::infoBar()->removeInfo(kLinkWithQtInstallationSetting);
-        QTimer::singleShot(0, ICore::dialogParent(), &QtOptionsPage::linkWithQt);
+        QTimer::singleShot(0, ICore::dialogParent(), &LinkWithQtSupport::linkWithQt);
     });
     ICore::infoBar()->addInfo(info);
 }
@@ -148,7 +210,7 @@ void QtSupportPlugin::extensionsInitialized()
         });
 
     static const auto activeQtVersion = []() -> const QtVersion * {
-        ProjectExplorer::Project *project = SessionManager::startupProject();
+        ProjectExplorer::Project *project = ProjectManager::startupProject();
         if (!project || !project->activeTarget())
             return nullptr;
         return QtKitAspect::qtVersion(project->activeTarget()->kit());
@@ -175,7 +237,7 @@ void QtSupportPlugin::extensionsInitialized()
 
     expander->registerVariable(
         "ActiveProject::QT_HOST_LIBEXECS",
-        Tr::tr("Full path to the libexec bin directory of the Qt version in the active kit "
+        Tr::tr("Full path to the libexec directory of the Qt version in the active kit "
                "of the active project."),
         []() {
             const QtVersion *const qt = activeQtVersion();
@@ -186,7 +248,7 @@ void QtSupportPlugin::extensionsInitialized()
         const FilePath filePath = item.filePath();
         if (filePath.isEmpty())
             return links;
-        const Project *project = SessionManager::projectForFile(filePath);
+        const Project *project = ProjectManager::projectForFile(filePath);
         Target *target = project ? project->activeTarget() : nullptr;
         QtVersion *qt = target ? QtKitAspect::qtVersion(target->kit()) : nullptr;
         if (!qt)
@@ -232,5 +294,6 @@ void QtSupportPlugin::extensionsInitialized()
     askAboutQtInstallation();
 }
 
-} // Internal
-} // QtSupport
+} // QtSupport::Internal
+
+#include "qtsupportplugin.moc"
