@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "callgrindproxymodel.h"
 
@@ -32,10 +10,7 @@
 
 #include <utils/qtcassert.h>
 
-#include <QDebug>
-
-namespace Valgrind {
-namespace Callgrind {
+namespace Valgrind::Callgrind {
 
 DataProxyModel::DataProxyModel(QObject *parent)
     : QSortFilterProxyModel(parent)
@@ -130,7 +105,8 @@ bool DataProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_
     // check if the function from this index is a child of (called by) the filter function
     if (m_function) {
         bool isValid = false;
-        foreach (const FunctionCall *call, func->incomingCalls()) {
+        const QList<const FunctionCall *> calls = func->incomingCalls();
+        for (const FunctionCall *call : calls) {
             if (call->caller() == m_function) {
                 isValid = true;
                 break;
@@ -156,5 +132,4 @@ bool DataProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_
     return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
 }
 
-} // namespace Callgrind
-} // namespace Valgrind
+} // namespace Valgrind::Callgrind
