@@ -1,41 +1,19 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Author: Frank Osterfeld, KDAB (frank.osterfeld@kdab.com)
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
+#include <QList>
 #include <QMetaType>
 #include <QSharedDataPointer>
 
 QT_BEGIN_NAMESPACE
 class QString;
-template <typename T> class QVector;
 QT_END_NAMESPACE
 
-namespace Valgrind {
-namespace XmlProtocol {
+namespace Valgrind::XmlProtocol {
+
+Q_NAMESPACE
 
 class Stack;
 class Suppression;
@@ -43,7 +21,7 @@ class Suppression;
 /**
  * Error kinds, specific to memcheck
  */
-enum MemcheckErrorKind
+enum MemcheckError
 {
     InvalidFree,
     MismatchedFree,
@@ -60,18 +38,20 @@ enum MemcheckErrorKind
     Leak_PossiblyLost,
     Leak_StillReachable,
     Leak_IndirectlyLost,
-    MemcheckErrorKindCount
+    ReallocSizeZero
 };
+Q_ENUM_NS(MemcheckError);
 
-enum PtrcheckErrorKind
+enum PtrcheckError
 {
     SorG,
     Heap,
     Arith,
     SysParam
 };
+Q_ENUM_NS(PtrcheckError);
 
-enum HelgrindErrorKind
+enum HelgrindError
 {
     Race,
     UnlockUnlocked,
@@ -81,6 +61,7 @@ enum HelgrindErrorKind
     LockOrder,
     Misc
 };
+Q_ENUM_NS(HelgrindError);
 
 class Error
 {
@@ -108,8 +89,8 @@ public:
     int kind() const;
     void setKind(int kind);
 
-    QVector<Stack> stacks() const;
-    void setStacks(const QVector<Stack> &stacks);
+    QList<Stack> stacks() const;
+    void setStacks(const QList<Stack> &stacks);
 
     Suppression suppression() const;
     void setSuppression(const Suppression &suppression);
@@ -132,7 +113,6 @@ private:
     QSharedDataPointer<Private> d;
 };
 
-} // namespace XmlProtocol
-} // namespace Valgrind
+} // namespace Valgrind::XmlProtocol
 
 Q_DECLARE_METATYPE(Valgrind::XmlProtocol::Error)
