@@ -1,12 +1,12 @@
 # Copyright (C) 2016 The Qt Company Ltd.
-# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 source("../../shared/qtcreator.py")
 
 def main():
     # prepare example project
     projectName = "adding"
-    sourceExample = os.path.join(Qt5Path.examplesPath(Targets.DESKTOP_5_14_1_DEFAULT),
+    sourceExample = os.path.join(QtPath.examplesPath(Targets.DESKTOP_5_14_1_DEFAULT),
                                  "qml", "referenceexamples", "adding")
     proFile = projectName + ".pro"
     if not neededFilePresent(os.path.join(sourceExample, proFile)):
@@ -79,7 +79,12 @@ def renameFile(projectDir, proFile, branch, oldname, newname):
         menu = ":Qt Creator.Project.Menu.Folder_QMenu"
     else:
         menu = ":Qt Creator.Project.Menu.File_QMenu"
-    activateItem(waitForObjectItem(menu, "Rename..."))
+    try:
+        activateItem(waitForObjectItem(menu, "Rename...", 5000))
+    except:
+        # Try getting an enabled item by reopening the menu
+        openItemContextMenu(treeview, oldItemText, 5, 5, 0)
+        activateItem(waitForObjectItem(menu, "Rename...", 5000))
     replaceEdit = waitForObject(":Qt Creator_Utils::NavigationTreeView::QExpandingLineEdit")
     test.compare(replaceEdit.selectedText, oldname.rsplit(".", 1)[0],
                  "Only the filename without the extension is selected?")
